/*
 * QuickTime for Java SDK Sample Code

   Usage subject to restrictions in SDK License Agreement
 * Copyright:  1996-1999 Apple Computer, Inc.

 */
package duke;

import java.awt.*;
import java.io.*;

import quicktime.io.QTFile;
import quicktime.util.QTUtils;
import quicktime.*;
import quicktime.app.*;
import quicktime.std.StdQTException;
import quicktime.std.movies.MovieController;
import quicktime.app.display.QTCanvas;

/**
 * A simple panel of AWT objects to demonstrate how AWT can be used to control a
 * QuickTime movie.
 */
public class ControlPanel extends Panel implements Errors {
//_________________________ CLASS METHODS
	public ControlPanel(Color bgColor, MovieScreen ms) {
		this.ms = ms;
	
		setBackground(bgColor);
		setLayout(new FlowLayout(FlowLayout.CENTER, 10, 3));
		setFont(new Font("Geneva", Font.BOLD, 10));
	
		// start, stop, loop and get a new movie
		movieButtons.setBackground(getBackground());
		movieButtons.setLayout(new GridLayout(4, 1, 3, 3));
		movieButtons.add(startButton);
		movieButtons.add(stopButton);
		loopButton.setState(false);
		movieButtons.add(loopButton);
		movieButtons.add(getMovieButton);
		add(movieButtons);
		
		// move movie up, down, left or right (can also use keypad keys)
/*		moveButtons.setBackground(getBackground());
		moveButtons.setLayout(new BorderLayout(3, 3));
		moveButtons.add("North", upButton);
		moveButtons.add("South", downButton);
		moveButtons.add("West", leftButton);
		moveButtons.add("East", rightButton);
		moveButtons.add ("Center", centerButton);
		add(moveButtons);
*/		
		// choose a resizing option
		resizeButtons.setBackground(getBackground());
		resizeButtons.setLayout(new GridLayout(4, 2));
		resizeButtons.add(initButton);
		resizeButtons.add(integralButton);
		resizeButtons.add(horizontalButton);
		resizeButtons.add(aspectButton);
		resizeButtons.add(verticalButton);
		resizeButtons.add(freeButton);
		resizeButtons.add(performButton);
		add(resizeButtons);

		// choose a visibility option
		visiblity.setBackground(getBackground());
		visiblity.setLayout(new GridLayout(3, 1));
		visiblity.add(visibleButton);
		visiblity.add(invisibleButton);
		add(visiblity);
	}

	public Dimension getPreferredSize () { return new Dimension (100, 100); }
//_________________________ INSTANCE VARIABLES
	private MovieScreen ms;

	private Panel  movieButtons   = new Panel();
	private Button startButton    = new Button("Start");
	private Button stopButton     = new Button("Stop");
	private Button getMovieButton = new Button("Get QT Media");

	private Checkbox loopButton = new Checkbox("Looping");

	private Panel         resizeButtons = new Panel();
	private CheckboxGroup cbg           = new CheckboxGroup();
	private Checkbox      initButton    = new Checkbox("Initial Size or Smaller", cbg, true);
	private Checkbox      freeButton    = new Checkbox("Allow Any Size", cbg, false);
	private Checkbox      aspectButton  = new Checkbox("Maintain Aspect", cbg, false);
	private Checkbox      performButton = new Checkbox("Performance Sizing", cbg, false);
	private Checkbox      horizontalButton = new Checkbox("Horizontal Sizing", cbg, false);
	private Checkbox      verticalButton = new Checkbox("Vertical Sizing", cbg, false);
	private Checkbox      integralButton = new Checkbox("Integral Sizing", cbg, false);

	private Panel         visiblity		 = new Panel();
	private CheckboxGroup cbg2           = new CheckboxGroup();
	private Checkbox      visibleButton  = new Checkbox("Visible", cbg2, true);
	private Checkbox      invisibleButton= new Checkbox("Invisible", cbg2, false);

//	private Panel  moveButtons = new Panel();
//	private Button upButton    = new Button("Up");
//	private Button downButton  = new Button("Down");
//	private Button leftButton  = new Button("Left");
//	private Button rightButton = new Button("Right");
//	private Button centerButton = new Button("Center");

//_________________________ INSTANCE METHODS
	public void doGetMovie () {
		try {
			ms.stopPlayer ();
			QTFile qtf = QTFile.standardGetFilePreview(QTFile.kStandardQTFileTypes);
// Below is code to use the Standard Java File Dialog, i.e no fancy Quicktime preview dialog.
//			FileDialog fd = new FileDialog ((Frame)getParent(), "Choose a QuickTime file to view:");
//			QTFileTypesFilter f = new QTFileTypesFilter();
//			f.set3DAccept(false);
//			fd.setFilenameFilter(f);
//			fd.show();
//			String fileDir = fd.getDirectory();
//			String fileName = fd.getFile();
//			if (fileDir != null && fileName != null) {
//				QTFile qtf = new QTFile (fileDir + fileName);
				ms.getNewMovie(qtf);
//			}
		} catch (QTException e) {
		} catch (Exception e) {
		} finally { QTUtils.reclaimMemory(); }
	}
	
	public void doMove (float deltaX, float deltaY) {
		QTCanvas canv = ms.getCanvas();
		if (canv == null) return;	
		float x = canv.getAlignmentX() + deltaX;
		float y = canv.getAlignmentY() + deltaY;
		if (x < 0) x = 1;
		else if (x > 1) //catch floating point errors
			if (x < 1.09) x = 1; else x = 0;
		if (y < 0) y = 1;
		else if (y > 1) //catch floating point errors
			if (y < 1.09) y = 1; else y = 0;
		ms.getCanvas().setAlignment(x, y);
	}
	
	/** Handle all the AWT button events here. */
	public boolean handleEvent(Event event) {
		if (event.id == Event.ACTION_EVENT) {
			if (event.target == getMovieButton) {
				doGetMovie();
				return true;
			}
			else if (event.target == visibleButton) {
				ms.getCanvas().setVisible(true);
				return true;
			}
			else if (event.target == invisibleButton) {
				ms.stopPlayer();
				ms.getCanvas().setVisible(false);
				return true;
			}
			else if (event.target == stopButton) {
				ms.stopPlayer();
				return true;
			}
			else if (event.target == startButton) {
				ms.startPlayer();
				return true;
			}
/*			else if (event.target == upButton) {
				doMove (0, -0.1F);
				return true;
			}
			else if (event.target == downButton) {
				doMove (0, 0.1F);
				return true;
			}
			else if (event.target == leftButton) {
				doMove (-0.1F, 0);
				return true;
			}
			else if (event.target == rightButton) {
				doMove (0.1F, 0);
				return true;
			}
			else if (event.target == centerButton) {
				QTCanvas canv = ms.getCanvas();
				if (canv == null) return true;	
				ms.getCanvas().setAlignment(0.5F, 0.5F);
				return true;
			}
*/			else if (event.target == initButton) {
				ms.getCanvas().setResizeFlag(QTCanvas.kInitialSize);
				return true;
			}
			else if (event.target == freeButton) {
				ms.getCanvas().setResizeFlag(QTCanvas.kFreeResize);
				return true;
			}
			else if (event.target == aspectButton) {
				ms.getCanvas().setResizeFlag(QTCanvas.kAspectResize);
				return true;
			}
			else if (event.target == performButton) {
				ms.getCanvas().setResizeFlag(QTCanvas.kPerformanceResize);
				return true;
			}
			else if (event.target == horizontalButton) {
				ms.getCanvas().setResizeFlag(QTCanvas.kHorizontalResize);
				return true;
			}
			else if (event.target == verticalButton) {
				ms.getCanvas().setResizeFlag(QTCanvas.kVerticalResize);
				return true;
			}
			else if (event.target == integralButton) {
				ms.getCanvas().setResizeFlag(QTCanvas.kIntegralResize);
				return true;
			}
			else if (event.target == loopButton) {
				if (ms.isLooping()) {
					loopButton.setState(false);
					ms.setLooping(false);
				}
				else {
					loopButton.setState(true);
					ms.setLooping(true);
				}
				return true;
			}
		}
		
		return super.handleEvent(event);
	}
}
