/*
 * QuickTime for Java SDK Sample Code

   Usage subject to restrictions in SDK License Agreement
 * Copyright:  1996-1999 Apple Computer, Inc.

 */
package duke;

import java.awt.*;
import java.awt.event.*;
import java.io.IOException;
import quicktime.QTException;
import quicktime.io.*;
import quicktime.std.movies.*;
import quicktime.app.players.*;
import quicktime.app.display.*;
import quicktime.app.image.*;
import quicktime.util.*;
import quicktime.app.QTFactory;
/**
 * This class is just used as a central location for the player, canvas and
 * controller, which are all set up via the getNewMovie method.
 */
public class MovieScreen {
//_________________________ CLASS METHODS
	public MovieScreen() throws QTException {
		canvas  = new QTCanvas(QTCanvas.kInitialSize, 0.5F, 0.5F);
		
		canvas.addQTClientListener (new ComponentAdapter () {
		    public void componentResized(ComponentEvent e) {
		    	System.out.println (e);
		    }
		    public void componentMoved(ComponentEvent e) {
		    	System.out.println (e);
		    }

		    public void componentShown(ComponentEvent e) {
		    	System.out.println (e);
		    }

		    public void componentHidden(ComponentEvent e) {
		    	System.out.println (e);
		    }
		});
		canvas.setClient (ImageDrawer.getQTLogo(), true);
	}
	
//_________________________ INSTANCE VARIABLES
	private QTCanvas canvas;
	private QTDrawable qtp = null;
	
//_________________________ INSTANCE METHODS
	/** Stops the player and removes it from the task thread*/
	public void stopPlayer () {
		try {
			if (qtp != null && qtp instanceof Playable)
				((Playable)qtp).setRate(0);
		} catch (QTException e) {}
	}

	/** Return the current QTCanvas. */
	public QTCanvas getCanvas() { return this.canvas; }
	
	public void startPlayer() {
		try {
			if (qtp != null && qtp instanceof Playable)
				((Playable)qtp).setRate(1);
		} catch (QTException e) {}
	}
	
	public void setLooping (boolean flag) {
		try {
			if (qtp != null && qtp instanceof Playable)
				((QTPlayer)qtp).getMovieController().setLooping(flag);	
		} catch (QTException e) {}
	}
	
	public boolean isLooping () {
		try {
			if (qtp != null && qtp instanceof Playable)
				return ((QTPlayer)qtp).getMovieController().getLooping();
		} catch (QTException e) {}
		return false;
	}

	/**
	 * Allow user to pick a new movie. Ordinarily, this method would return a
	 * boolean. But it returns the path string of the chosen movie so we know
	 * where to get the images for the tumbling duke animation.
	 */
	public void getNewMovie (QTFile qtf) throws QTException, IOException {
		stopPlayer();
		
		qtp = (QTFactory.makeDrawable(qtf));
		if (qtp instanceof QTPlayer)
			((QTPlayer)qtp).getMovieController().setKeysEnabled(true);
		
		// set the client of the current screen.
		canvas.setClient (qtp, true);
	}
}
