/*
 * QuickTime for Java SDK Sample Code

   Usage subject to restrictions in SDK License Agreement
 * Copyright:  1996-1999 Apple Computer, Inc.

 */
package mixer.mc;

import java.awt.*;
import javax.swing.*;
import quicktime.QTException;
import quicktime.app.audio.*;

/** This interface is the heart of the mixer.  The *Display classes define
  * what controls are created to mix a movie, and Displays can be created
  * out of any object that satisfies this interface.  Basically, any object
  * that implements this interface and has an appropriate subclass of
  * ChannelDisplay, can be mixed.
  */
public interface MixerComponents {
	/** This method returns the master control for the this channel.  This
	 *  is what is used to control the attributes of this channel.
	 *  @return the AudioSpec object that controls this channel
	 */
	public AudioSpec getMaster();
	
	/** This method returns an array of MixerComponent objects that represents
	 *  the "children" channels of this channel.  In a Music track, for example,
	 *  this might be each of the instrument channels that constitute the
	 *  Music track.
	 *  @return the array of MixerComponents objects that are the channels of this
	 *  object.
	 */
	public MixerComponents[] getChannels();
	
	/** This method defines how the channels of this object will be editted.  When
	 *  the user clicks the Edit button on the control channel made for this
	 *  object, this method will be invoked.  Whatever component this method returns
	 *  will determine the look of the next control panel.
	 *  @return the JComponent which contains the controls for the channels
	 */
	public JComponent makeEditComponent () throws QTException;

	/** Returns whether or not it's possible to edit this object further.
	 *  @return can this object be editted further?
	 */
	public boolean isEditable();
}
