#include <Endian.h>
#include <ImageCodec.h>

#include "EI_CodecVersion.h"
#include "EI_Image.h"

// data structures

typedef struct	{
	ComponentInstance	self;
	ComponentInstance	delegateComponent;
	ComponentInstance	target;
} EI_GlobalsRecord, *EI_Globals;

typedef struct {
	long		width;
	long		height;
	long		depth;
} EI_DecompressRecord;

// prototypes

static OSErr EI_Decompress1RLE(UInt8 *dataPtr, ICMDataProcRecordPtr dataProc, UInt8 *baseAddr, long rowBump, long width, long height);
static OSErr EI_Decompress8RLE(UInt8 *dataPtr, ICMDataProcRecordPtr dataProc, UInt8 *baseAddr, long rowBump, long width, long height);
static OSErr EI_Decompress16RLE(UInt8 *dataPtr, ICMDataProcRecordPtr dataProc, UInt8 *baseAddr, long rowBump, long width, long height);
static OSErr EI_Decompress24RLE(UInt8 *dataPtr, ICMDataProcRecordPtr dataProc, UInt8 *baseAddr, long rowBump, long width, long height);
static OSErr EI_Decompress32RLE(UInt8 *dataPtr, ICMDataProcRecordPtr dataProc, UInt8 *baseAddr, long rowBump, long width, long height);

// dispatcher

#define IMAGECODEC_BASENAME() 		EI_CD
#define IMAGECODEC_GLOBALS() 		EI_Globals storage

#define CALLCOMPONENT_BASENAME()		IMAGECODEC_BASENAME()
#define	CALLCOMPONENT_GLOBALS()			IMAGECODEC_GLOBALS()

#define COMPONENT_UPP_PREFIX()			uppImageCodec
#define COMPONENT_DISPATCH_FILE			"EI_CodecDispatch.h"
#define COMPONENT_SELECT_PREFIX()  		kImageCodec

#define	GET_DELEGATE_COMPONENT()		(storage->delegateComponent)

#include "Components.k.h"
#include "ImageCodec.k.h"
#include "ComponentDispatchHelper.c"

// code

pascal ComponentResult EI_CDOpen(EI_Globals glob, ComponentInstance self)
{
	ComponentResult err;

	glob = (EI_Globals)NewPtrClear(sizeof(EI_GlobalsRecord));
	if (err = MemError()) goto bail;

	SetComponentInstanceStorage(self, (Handle)glob);

	glob->self = self;
	glob->target = self;

	err = OpenADefaultComponent(decompressorComponentType, kBaseCodecType, &glob->delegateComponent);
	if (err) goto bail;

	ComponentSetTarget(glob->delegateComponent, self);

bail:
	return err;
}

pascal ComponentResult EI_CDClose(EI_Globals glob, ComponentInstance self)
{
	if (glob) {
		if (glob->delegateComponent)
			CloseComponent(glob->delegateComponent);

		DisposePtr((Ptr)glob);
	}

	return noErr;
}

pascal ComponentResult EI_CDTarget(EI_Globals glob, ComponentInstance target)
{
	glob->target = target;

	return noErr;
}

pascal ComponentResult EI_CDVersion(EI_Globals glob)
{
#pragma unused(glob)
#if TARGET_CPU_PPC
	return kEI_CodecVersionPPC;
#else
	return kEI_CodecVersion;
#endif
}

pascal ComponentResult EI_CDInitialize(EI_Globals glob, ImageSubCodecDecompressCapabilities *cap)
{
#pragma unused(glob)
	cap->decompressRecordSize = sizeof(EI_DecompressRecord);
	cap->canAsync = true;

	return noErr;
}

pascal ComponentResult EI_CDPreflight(EI_Globals glob, CodecDecompressParams *p)
{
#pragma unused(glob)
	CodecCapabilities *capabilities = p->capabilities;

	capabilities->bandMin = (**p->imageDescription).height;
	capabilities->bandInc = capabilities->bandMin;

	capabilities->wantedPixelSize = (**p->imageDescription).depth;
	if (capabilities->wantedPixelSize == 24)
		capabilities->wantedPixelSize = 32;

	capabilities->extendWidth = 0;
	capabilities->extendHeight = 0;

	return noErr;
}

pascal ComponentResult EI_CDBeginBand(EI_Globals glob, CodecDecompressParams *p, ImageSubCodecDecompressRecord *drp, long flags)
{
#pragma unused(glob)
	long offsetH, offsetV;
	EI_DecompressRecord *myDrp = (EI_DecompressRecord *)drp->userDecompressRecord;

	offsetH = (long)(p->dstRect.left - p->dstPixMap.bounds.left) * (long)(p->dstPixMap.pixelSize >> 3);
	offsetV = (long)(p->dstRect.top - p->dstPixMap.bounds.top) * (long)drp->rowBytes;

	drp->baseAddr = p->dstPixMap.baseAddr + offsetH + offsetV;

	myDrp->width = (**p->imageDescription).width;
	myDrp->height = (**p->imageDescription).height;
	myDrp->depth = (**p->imageDescription).depth;

	return noErr;
}

pascal ComponentResult EI_CDDrawBand(EI_Globals glob, ImageSubCodecDecompressRecord *drp)
{
#pragma unused(glob)
	OSErr err = noErr;
	EI_DecompressRecord *myDrp = (EI_DecompressRecord *)drp->userDecompressRecord;
	SInt8 mmuMode = true32b;
	unsigned char *dataPtr = (unsigned char *)drp->codecData;
	ICMDataProcRecordPtr dataProc = drp->dataProcRecord.dataProc ? &drp->dataProcRecord : nil;
	ImageFramePtr framePtr = (ImageFramePtr)dataPtr;

	// skip the Image Frame Header and palette
	dataPtr += sizeof(ImageFrame);
	dataPtr += EndianU16_BtoN(framePtr->framePalettes) * 3;

	SwapMMUMode(&mmuMode);

	if (framePtr->framePacking == 0) {
		// uncompressed data
	}
	else if (framePtr->framePacking == 1) {
		switch (myDrp->depth) {
			case 1:
				err = EI_Decompress1RLE(dataPtr, dataProc, (UInt8 *)drp->baseAddr, drp->rowBytes - ((myDrp->width + 7) / 8), myDrp->width, myDrp->height);
				break;

			case 8:
				err = EI_Decompress8RLE(dataPtr, dataProc, (UInt8 *)drp->baseAddr, drp->rowBytes - (myDrp->width * 1), myDrp->width, myDrp->height);
				break;

			case 16:
				err = EI_Decompress16RLE(dataPtr, dataProc, (UInt8 *)drp->baseAddr, drp->rowBytes - (myDrp->width * 2), myDrp->width, myDrp->height);
				break;

			case 24:
				err = EI_Decompress24RLE(dataPtr, dataProc, (UInt8 *)drp->baseAddr, drp->rowBytes - (myDrp->width * 4), myDrp->width, myDrp->height);
				break;

			case 32:
				err = EI_Decompress32RLE(dataPtr, dataProc, (UInt8 *)drp->baseAddr, drp->rowBytes - (myDrp->width * 4), myDrp->width, myDrp->height);
				break;
		}
	}

bail:
	SwapMMUMode(&mmuMode);

	return err;
}

pascal ComponentResult EI_CDEndBand(EI_Globals glob, ImageSubCodecDecompressRecord *drp, OSErr result, long flags)
{
#pragma unused(glob, drp,result, flags)
	return noErr;
}

pascal ComponentResult EI_CDQueueStarting(EI_Globals glob)
{
#pragma unused(glob)
	return noErr;
}

pascal ComponentResult EI_CDQueueStopping(EI_Globals glob)
{
#pragma unused(glob)
	return noErr;
}

pascal ComponentResult EI_CDGetCompressedImageSize(EI_Globals glob, ImageDescriptionHandle desc, Ptr data, long dataSize, ICMDataProcRecordPtr dataProc, long *size)
{
#pragma	unused(glob,dataSize,dataProc)
	ImageFramePtr framePtr = (ImageFramePtr)data;

	if (size == nil) 
		return paramErr;

	*size = EndianU32_BtoN(framePtr->frameSize) + sizeof(ImageFrame);

	return noErr;
}

pascal ComponentResult EI_CDGetCodecInfo(EI_Globals glob, CodecInfo *info)
{
	OSErr err = noErr;

	if (info == nil) {
		err = paramErr;
	}
	else {
		CodecInfo **tempCodecInfo;

		err = GetComponentResource((Component)glob->self, codecInfoResourceType, 256, (Handle *)&tempCodecInfo);
		if (err == noErr) {
			*info = **tempCodecInfo;
			DisposeHandle((Handle)tempCodecInfo);
		}
	}

	return err;
}


// Someday you may find yourself on an architecture where misaligned reads
// are ultra-expensive. On that day you can cheerfully adjust these macros to compensate.

#define Get32(x)		(*(long*)(x))
#define GetU32(x)		(*(unsigned long*)(x))
#define Set32(x,y)		(*(long*)(x)) = ((long)(y))

#define Get16(x)		(*(short*)(x))
#define GetU16(x)		(*(unsigned short*)(x))
#define Set16(x,y)		(*(short*)(x)) = ((short)(y))


#define kSpoolChunkSize (16384)
#define kInfiniteDataSize (0x7fffffff)

OSErr EI_Decompress32RLE(UInt8 *dataPtr, ICMDataProcRecordPtr dataProc, UInt8 *baseAddr, long rowBump, long width, long height)
{
	OSErr err = noErr;
	UInt8 *endOfScanLine;
	long availableData = dataProc ? codecMinimumDataSize : kInfiniteDataSize;

	endOfScanLine = baseAddr + (width * 4);

	while (height) {
		UInt16 opcode;

		if (availableData < kSpoolChunkSize) {
			// get some more source data
			err = CallICMDataProc(dataProc->dataProc, (Ptr *)&dataPtr, codecMinimumDataSize, dataProc->dataRefCon);
			if (err == eofErr)
				err = noErr;
			if (err) goto bail;

			availableData = codecMinimumDataSize;
		}

		opcode = *dataPtr++;
		if (opcode <= 127) {
			// repeat
			UInt32 pixel = GetU32(dataPtr);
			dataPtr += sizeof(UInt32);
			opcode++;
			availableData -= 5;
			while (opcode--) {
				Set32(baseAddr, pixel);
				baseAddr += sizeof(long);
				if (baseAddr == endOfScanLine) {
					baseAddr += rowBump;
					endOfScanLine = baseAddr + (width * 4);
					height--;
				}
			}
		}
		else {
			// quote
			opcode -= 127;
			availableData -= (1 + (opcode * 4));
			while (opcode--) {
				Set32(baseAddr, GetU32(dataPtr));
				baseAddr += sizeof(long);
				dataPtr += sizeof(long);
				if (baseAddr == endOfScanLine) {
					baseAddr += rowBump;
					endOfScanLine = baseAddr + (width * 4);
					height--;
				}
			}
		}
	}

bail:
	return err;
}

OSErr EI_Decompress24RLE(UInt8 *dataPtr, ICMDataProcRecordPtr dataProc, UInt8 *baseAddr, long rowBump, long width, long height)
{
	OSErr err = noErr;
	UInt8 *endOfScanLine;
	long availableData = dataProc ? codecMinimumDataSize : kInfiniteDataSize;

	endOfScanLine = baseAddr + (width * 4);

	while (height) {
		UInt16 opcode;

		if (availableData < kSpoolChunkSize) {
			// get some more source data
			err = CallICMDataProc(dataProc->dataProc, (Ptr *)&dataPtr, codecMinimumDataSize, dataProc->dataRefCon);
			if (err == eofErr)
				err = noErr;
			if (err) goto bail;

			availableData = codecMinimumDataSize;
		}

		opcode = *dataPtr++;
		if (opcode <= 127) {
			// repeat
#if TARGET_RT_BIG_ENDIAN
			UInt32 pixel = GetU32(dataPtr) >> 8;
#else
			UInt32 pixel = GetU32(dataPtr) << 8;
#endif
			dataPtr += 3;
			opcode++;
			availableData -= 4;
			while (opcode--) {
				Set32(baseAddr, pixel);
				baseAddr += sizeof(long);
				if (baseAddr == endOfScanLine) {
					baseAddr += rowBump;
					endOfScanLine = baseAddr + (width * 4);
					height--;
				}
			}
		}
		else {
			// quote
			opcode -= 127;
			availableData -= (1 + (opcode * 3));
			while (opcode--) {
#if TARGET_RT_BIG_ENDIAN
				UInt32 pixel = GetU32(dataPtr) >> 8;
#else
				UInt32 pixel = GetU32(dataPtr) << 8;
#endif
				Set32(baseAddr, pixel);
				baseAddr += sizeof(UInt32);
				dataPtr += 3;
				if (baseAddr == endOfScanLine) {
					baseAddr += rowBump;
					endOfScanLine = baseAddr + (width * 4);
					height--;
				}
			}
		}
	}

bail:
	return err;
}

OSErr EI_Decompress16RLE(UInt8 *dataPtr, ICMDataProcRecordPtr dataProc, UInt8 *baseAddr, long rowBump, long width, long height)
{
	OSErr err = noErr;
	UInt8 *endOfScanLine;
	long availableData = dataProc ? codecMinimumDataSize : kInfiniteDataSize;

	endOfScanLine = baseAddr + (width * 2);

	while (height) {
		UInt16 opcode;

		if (availableData < kSpoolChunkSize) {
			// get some more source data
			err = CallICMDataProc(dataProc->dataProc, (Ptr *)&dataPtr, codecMinimumDataSize, dataProc->dataRefCon);
			if (err == eofErr)
				err = noErr;
			if (err) goto bail;

			availableData = codecMinimumDataSize;
		}

		opcode = *dataPtr++;
		if (opcode <= 127) {
			// repeat
			UInt16 pixel = GetU16(dataPtr);
			dataPtr += sizeof(UInt16);
			opcode++;
			availableData -= 3;
			while (opcode--) {
				Set16(baseAddr, pixel);
				baseAddr += sizeof(UInt16);
				if (baseAddr == endOfScanLine) {
					baseAddr += rowBump;
					endOfScanLine = baseAddr + (width * 2);
					height--;
				}
			}
		}
		else {
			// quote
			opcode -= 127;
			availableData -= (1 + (opcode * 2));
			while (opcode--) {
				Set16(baseAddr, GetU16(dataPtr));
				baseAddr += sizeof(UInt16);
				dataPtr += sizeof(UInt16);
				if (baseAddr == endOfScanLine) {
					baseAddr += rowBump;
					endOfScanLine = baseAddr + (width * 2);
					height--;
				}
			}
		}
	}

bail:
	return err;
}

OSErr EI_Decompress8RLE(UInt8 *dataPtr, ICMDataProcRecordPtr dataProc, UInt8 *baseAddr, long rowBump, long width, long height)
{
	OSErr err = noErr;
	UInt8 *endOfScanLine;
	long availableData = dataProc ? codecMinimumDataSize : kInfiniteDataSize;

	endOfScanLine = baseAddr + (width * 1);

	while (height) {
		UInt16 opcode;

		if (availableData < kSpoolChunkSize) {
			// get some more source data
			err = CallICMDataProc(dataProc->dataProc, (Ptr *)&dataPtr, codecMinimumDataSize, dataProc->dataRefCon);
			if (err == eofErr)
				err = noErr;
			if (err) goto bail;

			availableData = codecMinimumDataSize;
		}

		opcode = *dataPtr++;
		if (opcode <= 127) {
			// repeat
			UInt8 pixel = *(UInt8 *)dataPtr;
			dataPtr += sizeof(UInt8);
			opcode++;
			availableData -= 2;
			while (opcode--) {
				*(UInt8 *)baseAddr = pixel;
				baseAddr += sizeof(UInt8);
				if (baseAddr == endOfScanLine) {
					baseAddr += rowBump;
					endOfScanLine = baseAddr + (width * 1);
					height--;
				}
			}
		}
		else {
			// quote
			opcode -= 127;
			availableData -= (1 + (opcode * 1));
			while (opcode--) {
				*(UInt8 *)baseAddr = *(UInt8 *)dataPtr;
				baseAddr += sizeof(UInt8);
				dataPtr += sizeof(UInt8);
				if (baseAddr == endOfScanLine) {
					baseAddr += rowBump;
					endOfScanLine = baseAddr + (width * 1);
					height--;
				}
			}
		}
	}

bail:
	return err;
}

OSErr EI_Decompress1RLE(UInt8 *dataPtr, ICMDataProcRecordPtr dataProc, UInt8 *baseAddr, long rowBump, long width, long height)
{
	OSErr err = noErr;
	UInt8 *endOfScanLine;
	long availableData = dataProc ? codecMinimumDataSize : kInfiniteDataSize;

	endOfScanLine = baseAddr + ((width + 7) / 8);

	while (height) {
		UInt16 opcode;

		if (availableData < kSpoolChunkSize) {
			// get some more source data
			err = CallICMDataProc(dataProc->dataProc, (Ptr *)&dataPtr, codecMinimumDataSize, dataProc->dataRefCon);
			if (err == eofErr)
				err = noErr;
			if (err) goto bail;

			availableData = codecMinimumDataSize;
		}

		opcode = *dataPtr++;
		if (opcode <= 127) {
			// repeat
			UInt8 pixel = *(UInt8 *)dataPtr;
			dataPtr += sizeof(UInt8);
			opcode++;
			availableData -= 2;
			while (opcode--) {
				*(UInt8 *)baseAddr = pixel;
				baseAddr += sizeof(UInt8);
				if (baseAddr == endOfScanLine) {
					baseAddr += rowBump;
					endOfScanLine = baseAddr + ((width + 7) / 8);
					height--;
				}
			}
		}
		else {
			// quote
			opcode -= 127;
			availableData -= (1 + (opcode * 1));
			while (opcode--) {
				*(UInt8 *)baseAddr = *(UInt8 *)dataPtr;
				baseAddr += sizeof(UInt8);
				dataPtr += sizeof(UInt8);
				if (baseAddr == endOfScanLine) {
					baseAddr += rowBump;
					endOfScanLine = baseAddr + ((width + 7) / 8);
					height--;
				}
			}
		}
	}

bail:
	return err;
}

#ifndef STAND_ALONE

void EI_CodecRegister(void);
void EI_CodecRegister(void)

{
	ComponentDescription td;
	ComponentRoutineUPP componentEntryPoint = NewComponentRoutineProc(EI_CDComponentDispatch);

	td.componentType = decompressorComponentType;
	td.componentSubType = FOUR_CHAR_CODE('EIDI');
	td.componentManufacturer = kAppleManufacturer;
	td.componentFlags = codecInfoDoes32;
	td.componentFlagsMask = 0;

	RegisterComponent(&td,componentEntryPoint, 0, nil,nil, nil);
}
#endif
