//////////
//
//	File:		ComApplication.h
//
//	Contains:	Application-specific code for QuickTime movies demo.
//
//	Written by:	Tim Monroe
//				Based (heavily!) on the MovieShell code written by Apple DTS
//
//	Copyright:	 1994-1997 by Apple Computer, Inc., all rights reserved.
//
//	Change History (most recent first):
//
//	   <4>	 	09/11/97	rtm		merged MacApplication.h and WinApplication.h
//	   <3>	 	08/21/97	rtm		begun adding Windows support
//	   <2>	 	11/27/96	rtm		conversion to personal coding style
//	   <1>	 	12/21/94	khs		first file
//	   
//////////

#pragma once

//////////
//
// header files
//
//////////

#ifndef __QUICKTIMEVR__
#include <QuickTimeVR.h>
#endif

#ifndef __QTUtilities__
#include "QTUtilities.h"
#endif

#ifndef __QTVRUtilities__
#include "QTVRUtilities.h"
#endif

#include <TextUtils.h>
#include <string.h>

#if TARGET_OS_MAC
#include "MacFramework.h"
#include "AppConfiguration.h"
#endif	// TARGET_OS_MAC

#if TARGET_OS_WIN32
#include "WinFramework.h"
#endif	// TARGET_OS_WIN32

#include "ComResource.h"

//////////
//
// constants
//
//////////

//////////
//
// structures
//
//////////

// application-specific data
typedef struct {
	Movie						fMovie;				// the embedded movie to play
	GWorldPtr					fOffscreenGWorld;	// the offscreen graphics world used for imaging embedded movies
	PixMapHandle				fOffscreenPixMap;	// the pixmap of the offscreen graphics world
	GWorldPtr					fPrevBBufGWorld;	// the previous offscreen graphics world used for the back buffer
	Rect						fPrevBBufRect;		// the previous rectangle of the area of interest
	QTVRFloatPoint				fMovieCenter;		// the center in the panorama of the movie screen (in angles: x = pan; y = tilt)
	Rect						fMovieBox;			// the movie box
	float						fMovieScale;		// a scale factor for the movie rectangle
	float						fMovieWidth;		// the width (in radians) of the embedded movie
	Boolean						fUseOffscreenGWorld;// use the specified movie GWorld?
	Boolean						fUseMovieCenter;	// use the specified movie center?
	Boolean						fQTMovieHasSound;	// does the embedded movie have a sound track?
	Boolean						fCompositeMovie;	// does the embedded movie need to be composited?
	Boolean						fUseHideRegion;		// use the specified movie hide region?
	QTVRBackBufferImagingUPP	fBackBufferProc;	// a routine descriptor for our back buffer routine
	RGBColor					fChromaColor;		// the color for chroma key compositing
	RgnHandle					fHideRegion;		// the region that obscures the embedded movie
	MatrixRecord				fMovieMatrix;		// the matrix we use to (optionally) rotate the movie
	MatrixRecord				fOrigMovieMatrix;	// the movie's original matrix
	Boolean						fBackBufferIsHoriz;	// is the backbuffer oriented horizontally?
	ImageDescriptionHandle		fImageDesc;			// image description for DecompressSequenceFrameS
	ImageSequence				fImageSequence;		// image sequence for DecompressSequenceFrameS	
} ApplicationDataRecord, *ApplicationDataPtr, **ApplicationDataHdl;


// the function prototypes are in the file MacFramework.h or WinFramework.h