#include <Endian.h>

#include "EI_MakeImageDescription.h"

OSErr EI_MakeImageDescription(ImageFramePtr frame, long colorCount, UInt8 *color, ImageDescriptionHandle *descOut)
{
	OSErr err = noErr;
	ImageDescriptionHandle desc = nil;
	ImageDescriptionPtr idp;
	CTabHandle colors = nil;

	desc = (ImageDescriptionHandle)NewHandleClear(sizeof(ImageDescription));
	if (err = MemError()) goto bail;

	idp = *desc;
	idp->idSize = sizeof(ImageDescription);
	idp->cType = FOUR_CHAR_CODE('EIDI');
	idp->vendor = kAppleManufacturer;
	idp->frameCount = 1;
	idp->depth = frame->frameBitDepth + frame->frameAlpha;
	idp->clutID = -1;
	idp->hRes = 72L << 16;
	idp->vRes = 72L << 16;
	idp->width = EndianU16_BtoN(frame->frameRect.right) - EndianU16_BtoN(frame->frameRect.left);
	idp->height = EndianU16_BtoN(frame->frameRect.bottom) - EndianU16_BtoN(frame->frameRect.top);
	idp->version = 0;

	// make up a color table, if there is one
	if (colorCount > 2) {
		short i;
		long ctSeed = GetCTSeed();

		colors = (CTabHandle)NewHandleClear(sizeof(long) + sizeof(short) * 2 + colorCount * sizeof(ColorSpec));
		if (err = MemError()) goto bail;

		(**colors).ctFlags = 0;
		(**colors).ctSeed = ctSeed;
		(**colors).ctSize = colorCount - 1;

		for (i=0; i<colorCount; i++) {
			(**colors).ctTable[i].value = i;
			(**colors).ctTable[i].rgb.red = ((UInt16)(color[0]) << 8) | color[0];
			(**colors).ctTable[i].rgb.green = ((UInt16)(color[1]) << 8) | color[1];
			(**colors).ctTable[i].rgb.blue = ((UInt16)(color[2]) << 8) | color[2];

			color += 3;
		}
		
		err = SetImageDescriptionCTable(desc, colors);
		if (err) goto bail;
	}

bail:
	if (colors)
		DisposeCTable(colors);

	if (desc && err) {
		DisposeHandle((Handle)desc);
		desc = nil;
	}

	*descOut = desc;

	return err;
}
