import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.io.IOException;

import quicktime.*;
import quicktime.io.*;
import quicktime.qd.*;
import quicktime.std.*;
import quicktime.std.clocks.*;
import quicktime.std.StdQTConstants;
import quicktime.std.image.*;
import quicktime.std.movies.*;
import quicktime.std.movies.media.*;
import quicktime.std.qtcomponents.*;
import quicktime.app.*;
import quicktime.app.display.*;
import quicktime.app.players.*;
import quicktime.util.*;

public class ImportExport extends Frame implements StdQTConstants, IOConstants,  Errors {	
	public static void main (String args[]) {
		try { 
			QTSession.open();
			ImportExport ie = new ImportExport("Import Export");
			ie.pack();
			ie.show();
			ie.toFront();
		} catch (Exception e) {
			e.printStackTrace();
			QTSession.close();
		}
	}

	ImportExport (String title)	throws Exception {
		super (title);

		Button b = new Button ("Import Media");
		b.addActionListener (new ActionListener () {
			public void actionPerformed (ActionEvent ae) {
				importMedia();
			}
		});
		
		add (b, "North");
		Button b2 = new Button ("Reference Movie");
		b2.addActionListener (new ActionListener () {
			public void actionPerformed (ActionEvent ae) {
				makeReferenceMovie();
			}
		});
		add (b2, "Center");

		Button b3 = new Button ("Export Movie");
		b3.addActionListener (new ActionListener () {
			public void actionPerformed (ActionEvent ae) {
				exportMovie();
			}
		});
		add (b3, "South");

		addWindowListener(new WindowAdapter () {
			public void windowClosing (WindowEvent e) {
				QTSession.close();
				dispose();
			}

			public void windowClosed (WindowEvent e) { 
				System.exit(0);
			}
		});	   

	}
	
	QTCanvas myQTCanvas;
	
	void importMedia () {
		try {
			FileDialog ifd = new FileDialog (this, "Choose Media to Import...", FileDialog.LOAD);
			ifd.show();
			if (ifd.getFile() == null)
				return;
			QTFile importFile = new QTFile (ifd.getDirectory() + ifd.getFile());
			
				//import ANY media into QuickTime using the QTFactory.
				// the makeDrawable method will import graphics using the GraphicsImporter
				// and any other media type, the MovieImporter is used the the Movie.fromFile call
				// The returned object can be drawn in a QTCanvas
			QTDrawable media = QTFactory.makeDrawable (importFile);
			if (myQTCanvas == null) {
				myQTCanvas = new QTCanvas();
				add (myQTCanvas, "East");
			}
			myQTCanvas.setClient (media, true);
				//resizes the frame to the size of the new QTCanvas and buttons.
				// the QTCanvas' size will be the size of the media
			pack();
		} catch (QTException err) {
			if (err.errorCode() == userCanceledErr) return;
			err.printStackTrace();
		} catch (IOException ie) {}
	}
	
	void displayMovie (Movie m) throws QTException {
			//make a QTPlayer out of the Movie and set it as the client of the QTCanvas
		QTPlayer p = new QTPlayer (new MovieController (m));
		if (myQTCanvas == null) {
			myQTCanvas = new QTCanvas();
			add (myQTCanvas, "East");
		}
		myQTCanvas.setClient (p, true);
		pack();
	}
			
	void makeReferenceMovie () {
		try{
			FileDialog rfd = new FileDialog (this, "Choose Movie to Reference...", FileDialog.LOAD);
			rfd.show();
			if (rfd.getFile() == null)
				return;
			QTFile movieFile = new QTFile (rfd.getDirectory() + rfd.getFile());

			FileDialog ofd = new FileDialog (this, "New Movie to create...", FileDialog.SAVE);
			ofd.show();
			if (ofd.getFile() == null) {
				return;
			}
			
			makeReferenceMovie (movieFile, ofd.getDirectory() + ofd.getFile());			
		} catch (QTException err) {
			if (err.errorCode() == userCanceledErr) return;
			err.printStackTrace();
		}
	}
		
		//makes a new movie that reference the data in an existing movie
	void makeReferenceMovie (QTFile movieFile, String outputPath) throws QTException {		
		// Create the movie object from the original movie
		Movie theMovie = Movie.fromFile (OpenMovieFile.asRead (movieFile));
		
		displayMovie (theMovie);
		
		QTFile outputMovie = new QTFile (outputPath);
		
		boolean creatingReferenceMovie = true;
			// this leaves data in existing movieFile
		if (creatingReferenceMovie) {
				//shortcut movies are movies that just contain a reference
				//to another movie. They are only available in QT 4 or above
			boolean createShortCutMovie = false;
			if (createShortCutMovie && QTSession.getQTMajorVersion() > 3) {
					//make a Data ref out of a URL that references the movie
				DataRef targetDataRef = new DataRef ("file://" + movieFile.getPath());
					//make the very small short cut movie
				outputMovie.createShortcutMovieFile (kMoviePlayer, smSystemScript, createMovieFileDeleteCurFile, targetDataRef);
			} else {
					// this create's a "medium" cut movie - same principle as the shortcut movie
					// it is a movie that gets its media data from another movie
				// create the new movie file, deleting existing file if there
				outputMovie.createMovieFile (kMoviePlayer, createMovieFileDeleteCurFile);
				
				// add a reference to all of the media in the existing media file
				// The outputMovie file now contains a reference and dependency on the existing media file
				OpenMovieFile outStream = OpenMovieFile.asWrite (outputMovie);
				theMovie.addResource (outStream, movieInDataForkResID, outputMovie.getName());
				outStream.close();
			}
		} else {
			// this will copy data to the new file
			
		//if the application wanted to remove this dependency it could flatten the existing movie
		// to the new movie file instead of the addResource calls that were made above
		// the createMovieFile call would not be required in this case
			int movieFlattenFlags = 0; //see docs to specify options here
			theMovie.flatten (movieFlattenFlags, 
									outputMovie, 
									kMoviePlayer,
									smSystemScript, 
									createMovieFileDeleteCurFile, 
									movieInDataForkResID, 
									outputMovie.getName());
		}
	}	

	void exportMovie () {
		try{
			FileDialog efd = new FileDialog (this, "Choose Movie to Export...", FileDialog.LOAD);
			efd.show();
			if (efd.getFile() == null)
				return;
			QTFile movieFile = new QTFile (efd.getDirectory() + efd.getFile());
			
			exportMovie (movieFile);
		
		} catch (QTException err) {
			err.printStackTrace();
		}
	}
	
		//export (to a movie) the incoming movie
		//user dialog allows user to customise media formats and tracks that are exported
	void exportMovie (QTFile movieFile) throws QTException {		
		// Create the movie object from the original movie
		Movie theMovie = Movie.fromFile (OpenMovieFile.asRead (movieFile));
		
		displayMovie (theMovie);
		
			//we do this in a different thread because exporting can take some time
			// and the event thread should not be blocked for so long
		new Thread (new Runner (theMovie)).start();
	}
	
	static class Runner implements Runnable {
		Runner (Movie mov) {
			theInputMovie = mov;
		}
		
		Movie theInputMovie;
		
		public void run () {
			try {
					//this determines both the exporter type, the resulting file type.
					// thus one could specify this to be AIFF and the resulting file will be an AIFF file
					// - in this case the result will be a movie.
				int exportType = kQTFileTypeMovie;
				
					//an application can alternatively configure exporter through setting
					// up the exporter in code to conform to the format appropriate
				
				boolean useMovieConvertFile = true;		
				if (useMovieConvertFile) {
						//set the progress proc of the movie so that it gives the user feedback
					theInputMovie.setProgressProc();
						//do the export of the movie - all tracks as specified by user in dialog
						// with the showUser... flag QT itself will display a dialog to the user
						// to save the exported file. The Name of the file in that dialog is the name we specify
						// in this case "Export Movie"
					theInputMovie.convertToFile (new QTFile ("Export Movie"), exportType, kMoviePlayer, IOConstants.smSystemScript, showUserSettingsDialog);
				} else {
					FileDialog ofd = new FileDialog (new Frame(), "Export Movie to...", FileDialog.SAVE);
					ofd.show();
					if (ofd.getFile() == null)
						return;
					QTFile outFile = new QTFile (ofd.getDirectory() + ofd.getFile());
					
					// Create a movie exporter so we can customise its settings
					// this could also be used in the convertToFile version, but
					// if we don't have custom settings then we allow the convertToFile
					// to create the exporter for us - based on the exportType we pass to it.
					MovieExporter theMovieExp = new MovieExporter (exportType);

					// Set export settings from the user.
					theMovieExp.doUserDialog (theInputMovie, null, 0, theInputMovie.getDuration());
					
					//this returns a dupFNErr on windows and is also more work for the application
						// create the output file but don't open it
					outFile.createMovieFile (kMoviePlayer, createMovieFileDeleteCurFile);
						//set a progress proc - we just print out to System.out
					theMovieExp.setProgressProc (new MovieProgress () {
					 	public int execute (Movie mov, int message, int whatOperation, float percentDone) {
							System.out.println (mov + ",message=" + Integer.toHexString(message) + "," + ",what=" + Integer.toHexString(whatOperation) + ",%=" + percentDone);
							return 0;
						}
					});
						// do the export of the movie
					theMovieExp.toFile (outFile, theInputMovie, null, 0, theInputMovie.getDuration());
				}
			} catch (QTException e) {
				e.printStackTrace();
			}
		}
	}
}