import java.awt.*;
import java.awt.event.*;

import java.util.Vector;
import java.util.StringTokenizer;

import quicktime.QTException;
import quicktime.QTSession;
import quicktime.app.display.QTCanvas;
import quicktime.app.anim.Compositor;
import quicktime.app.image.QTTransition;
import quicktime.std.movies.Atom;
import quicktime.std.movies.AtomContainer;
import quicktime.std.StdQTConstants;
import quicktime.qd.QDDimension;

import quicktime.util.EndianOrder;


/**
 * QTZoo Module 10 - Putting it all together 
 * This application requires QuickTime for Java
 *
 * @author Levi Brown
 * @author Michael Hopkins
 * @author Apple Computer, Inc.
 * @version 10.0 4/10/2000
 * 
 * Copyright: 	 Copyright 1999 Apple Computer, Inc. All rights reserved.
 *	
 * Disclaimer:	IMPORTANT:  This Apple software is supplied to you by Apple Computer, Inc.
 *				("Apple") in consideration of your agreement to the following terms, and your
 *				use, installation, modification or redistribution of this Apple software
 *				constitutes acceptance of these terms.  If you do not agree with these terms,
 *				please do not use, install, modify or redistribute this Apple software.
 *
 *				In consideration of your agreement to abide by the following terms, and subject
 *				to these terms, Apple grants you a personal, non-exclusive license, under Apples
 *				copyrights in this original Apple software (the "Apple Software"), to use,
 *				reproduce, modify and redistribute the Apple Software, with or without
 *				modifications, in source and/or binary forms; provided that if you redistribute
 *				the Apple Software in its entirety and without modifications, you must retain
 *				this notice and the following text and disclaimers in all such redistributions of
 *				the Apple Software.  Neither the name, trademarks, service marks or logos of
 *				Apple Computer, Inc. may be used to endorse or promote products derived from the
 *				Apple Software without specific prior written permission from Apple.  Except as
 *				expressly stated in this notice, no other rights or licenses, express or implied,
 *				are granted by Apple herein, including but not limited to any patent rights that
 *				may be infringed by your derivative works or by other works in which the Apple
 *				Software may be incorporated.
 *
 *				The Apple Software is provided by Apple on an "AS IS" basis.  APPLE MAKES NO
 *				WARRANTIES, EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION THE IMPLIED
 *				WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *				PURPOSE, REGARDING THE APPLE SOFTWARE OR ITS USE AND OPERATION ALONE OR IN
 *				COMBINATION WITH YOUR PRODUCTS.
 *
 *				IN NO EVENT SHALL APPLE BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL OR
 *				CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 *				GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *				ARISING IN ANY WAY OUT OF THE USE, REPRODUCTION, MODIFICATION AND/OR DISTRIBUTION
 *				OF THE APPLE SOFTWARE, HOWEVER CAUSED AND WHETHER UNDER THEORY OF CONTRACT, TORT
 *				(INCLUDING NEGLIGENCE), STRICT LIABILITY OR OTHERWISE, EVEN IF APPLE HAS BEEN
 *				ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

public class MainFrame extends Panel	
{
	public static final int WIDTH  = 640;
	public static final int HEIGHT = 480;

	/**
	 * Creates the QTCanvas, loads media, and displays window contents
	 */
	public MainFrame( Window window, Progress progress ) 
	{		
		this.window = window;
		this.progress = progress;
		progress.updateProgress("Setting sail...");
		setLayout(null);
		setSize( WIDTH, HEIGHT );
		
		myQTCanvas = new QTCanvas( QTCanvas.kInitialSize, 0.5F, 0.5F ); 	// Create new QT Canvas
		add( myQTCanvas );
		
		loadPanes( );	
		
		try
		{
			myQTCanvas.setClient (mapPane.getCompositor(), true);				//Set the canvas to display the map pane
		}
		catch (QTException exc)
		{
			exc.printStackTrace();
		}																	
		
		progress.updateProgress("Docking...");
	}
		
	/**
	 * Responsible for exiting the program
	 * Hides the window and shuts down QTJ
	 */
	public void handleQuit()
	{
		setVisible(false);
		window.setVisible(false);
		QTSession.close();
		System.exit(0);
	}
	
	/**
	 * Responds to requests to update regions of the screen
	 */
  	public void update( Graphics g )
  	{
  		paint(g);
  	}

	/**
     * Returns the preferred size of the frame
     */
	public Dimension getPreferredSize()
	{
		return new Dimension(WIDTH, HEIGHT);
	}

	
	/**
	 * Loads the data displayed in each of the animal panes
	 */
	public void loadPanes()
	{
		actionListener = new Action();

		mapPane = new MapPane( progress );
		mapPane.addActionListener(actionListener);

		progress.updateProgress("Azimuth determined.");
		
		zebraPane = new ZebraPane( progress );
		zebraPane.addActionListener(actionListener);

		progress.updateProgress("Zebras aboard.");
		
		giraffePane = new GiraffePane(progress);
		giraffePane.addActionListener(actionListener);
		progress.updateProgress("Giraffes aboard.");

		tigerPane = new TigerPane(progress);
		tigerPane.addActionListener(actionListener);
		progress.updateProgress("Tigers aboard.");

		rhinoPane = new RhinoPane(progress);
		rhinoPane.addActionListener(actionListener);
		progress.updateProgress("Rhinoceroses aboard.");

		bearPane = new BearPane(progress);
		bearPane.addActionListener(actionListener);
		progress.updateProgress("Kodiak loaded.");

		elephantPane = new ElephantPane(progress);
		elephantPane.addActionListener(actionListener);
		progress.updateProgress("Elephants settled in.");
	}
	
	protected void transition(ZooPane fromPane, ZooPane toPane, Point effectLoc )
	{
		Compositor fromComp = fromPane.getCompositor();
		Compositor toComp = toPane.getCompositor();
		try
		{
			//Make sure the compositor has had a chance to drawn itself
			toComp.setGWorld(quicktime.qd.QDGraphics.validScratch);
			toComp.redraw(null);
			
			QTTransition transition = new QTTransition(new QDDimension(MainFrame.WIDTH, MainFrame.HEIGHT));
			transition.setTime(1000);
			transition.setSourceImage(fromComp);
			transition.setDestinationImage(toComp);
			transition.setProfiled(false);
			myQTCanvas.setClient (transition, true);

			AtomContainer effectSample = new AtomContainer();

			if (effectLoc == null)
			{
			    // We are using SMPTE Effects so set the what atom to smpt
				effectSample.insertChild (Atom.kParentIsContainer, StdQTConstants.kEffectWhatAtom, 1, 0, EndianOrder.flipNativeToBigEndian32(StdQTConstants.kWipeTransitionType));		
			    // We are using SMPTE effect number 74	- start at 0%, stop at 100%
				effectSample.insertChild(Atom.kParentIsContainer, StdQTConstants.kEffectWipe, 1, 0, EndianOrder.flipNativeToBigEndian32(StdQTConstants.kRandomWipeTransitionType));
			}
			else
			{
				effectSample.insertChild(Atom.kParentIsContainer, StdQTConstants.kEffectWhatAtom, 1, 0, EndianOrder.flipNativeToBigEndian32(StdQTConstants.kExplodeTransitionType));		
				effectSample.insertChild(Atom.kParentIsContainer, StdQTConstants.kExplodeTransitionType, 1, 0, EndianOrder.flipNativeToBigEndian32(StdQTConstants.kExplodeTransitionType));
			}

			transition.setEffect(effectSample);
			transition.doTransition();

			myQTCanvas.setClient (toComp, true);

			toPane.start();
			fromPane.stop();
		}
		catch (QTException exc)
		{
			exc.printStackTrace();
		}
	}
	
    /**
	 * Responds to actions targeting the MainFrame
	 * Responsible for handling clicks in the animal regions in the Map Pane
	 */
	class Action implements ActionListener
	{
		public void actionPerformed(ActionEvent event)
		{
			Object source = event.getSource();
			String command = event.getActionCommand();
			StringTokenizer st = new StringTokenizer(command, ";");
			Vector params = new Vector();
			
			while (st.hasMoreTokens())
			{
				params.addElement(st.nextToken());
			}

			//Minimalistic error checking...
			if (params.size() < 3)
			{
				System.err.println("Not enough parameters in command");
				return;
			}
			
			//Parse our action command for pertinate info	
			String area = (String)params.elementAt(0);
			int xLoc = 0;
			int yLoc = 0;
			
			try
			{
				xLoc = Integer.parseInt((String)params.elementAt(1));
			} catch (NumberFormatException exc)
			{
				exc.printStackTrace();
			}
			try
			{
				yLoc = Integer.parseInt((String)params.elementAt(2));
			} catch (NumberFormatException exc)
			{
				exc.printStackTrace();
			}
				
			//Distribute appropriately
			
			if (source.equals(mapPane))
			{
				if (area.equals("zebra"))
				{
					zebraPane.playSound();
					transition(mapPane, zebraPane, new Point(xLoc, yLoc));
				}
				else if (area.equals("giraffe"))
				{
					giraffePane.playSound();
					transition(mapPane, giraffePane, new Point(xLoc, yLoc));
				}
				else if (area.equals("tiger"))
				{
					tigerPane.playSound();
					transition(mapPane, tigerPane, new Point(xLoc, yLoc));
				}
				else if (area.equals("rhino"))
				{
					rhinoPane.playSound();
					transition(mapPane, rhinoPane, new Point(xLoc, yLoc));
				}
				else if (area.equals("bear"))
				{
					bearPane.playSound();
					transition(mapPane, bearPane, new Point(xLoc, yLoc));
				}
				else if (area.equals("elephant"))
				{
					elephantPane.playSound();
					transition(mapPane, elephantPane, new Point(xLoc, yLoc));
				}			
				else if (area.equals("exit"))
				{
					handleQuit();
				}			
		}
			else if (source instanceof AnimalPane)
			{
				if (area.equals("map"))
				{
					transition((AnimalPane)source, mapPane, null );
				}
			}
		}
	}

	protected ActionListener actionListener;
	protected QTCanvas myQTCanvas;
	protected MapPane mapPane;
	protected ZebraPane zebraPane;

	protected GiraffePane giraffePane;
	protected TigerPane tigerPane;
	protected RhinoPane rhinoPane;
	protected BearPane bearPane;
	protected ElephantPane elephantPane;	
	
	protected Progress progress;

	protected Window window;
}
